#include <cutil.h>
#include <stdio.h>


/**
   Set this to 1 to trigger the bug.
   The program semantic is obviously the same, but we get a different result.
*/
#define TRIGGER_BUG 1


typedef int data_t;


struct Info
{
  data_t *p1, *p2, *p3;
};

__global__ void kernel(int i, data_t *g_data1, Info *g_info)
{
  data_t *g_data2 = g_data1 + i;
  data_t *g_data3 = g_data2 - (((long)g_data2 / sizeof(data_t)) & 0x3f);
  g_info->p1 = g_data1;
#if !TRIGGER_BUG
  g_info->p2 = g_data2;
#endif
  g_info->p3 = g_data3;
}

int
main()
{
  Info info;
  data_t *g_data1;
  Info *g_info;
  CUT_DEVICE_INIT();
  CUDA_SAFE_CALL(cudaMalloc((void **)&g_data1, 1024));
  CUDA_SAFE_CALL(cudaMalloc((void **)&g_info, sizeof(Info)));

  // start a single thread:
  dim3 grid(1, 1, 1);
  dim3 threads(1, 1, 1);
  kernel<<<grid, threads, 0>>>(0, g_data1, g_info);

  CUDA_SAFE_CALL(cudaMemcpy(&info, g_info, sizeof(Info), cudaMemcpyDeviceToHost));
  printf("%p %p\n", info.p1, info.p3);
}
